package com.rayfocus.sqs.app;

import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.model.DeleteMessageRequest;
import com.amazonaws.services.sqs.model.Message;
import com.amazonaws.services.sqs.model.ReceiveMessageRequest;
import com.amazonaws.services.sqs.model.ReceiveMessageResult;
import com.rayfocus.sqs.client.SQSClientProvider;
import com.rayfocus.sqs.util.SQSUtil;

import java.util.Date;
import java.util.List;

public class Consumer {

    private static final String SQS_INPUTQ = "SQS_INPUTQ";

    public static void main(String[] args) {
        Consumer consumer = new Consumer();
        // Receive messages from SQS_INPUTQ
        //consumer.receiveMessagesFrom(SQS_INPUTQ);
        consumer.receiveBatchMessages(SQS_INPUTQ);
    }

    /* Receive messages from Queue */
    public void receiveMessagesFrom(String queueName){
        AmazonSQS sqsClient = SQSClientProvider.getForProfile("sqs_demo");
        String queueUrl = SQSUtil.getQueueUrlFor(queueName);
        System.out.println("Start - receive messages");
        ReceiveMessageRequest receiveMessageRequest = new ReceiveMessageRequest(queueUrl)
                .withMaxNumberOfMessages(3);
        ReceiveMessageResult receiveMessageResult = sqsClient.receiveMessage(receiveMessageRequest);
        List<Message> messages = receiveMessageResult.getMessages();
        if(messages.isEmpty()){
            System.out.println("No messages available");
        }
        for(Message msg: messages){
            System.out.println(msg.getBody());
            // From here message can be sent further to external systems
        }
    }

    /* Receive multiple messages from Queue */
    public void receiveBatchMessages(String queueName){
        AmazonSQS sqsClient = SQSClientProvider.getForProfile("sqs_demo");
        String queueUrl = SQSUtil.getQueueUrlFor(queueName);
        System.out.println("Start - receive messages");
        int pollcount = 1;
        while(true){
            System.out.println("\n*************Polling attempt************* on "+new Date()+"\n");
            ReceiveMessageRequest receiveMessageRequest = new ReceiveMessageRequest(queueUrl)
                    .withMaxNumberOfMessages(2)
                    .withWaitTimeSeconds(5);
            ReceiveMessageResult receiveMessageResult = sqsClient.receiveMessage(receiveMessageRequest);
            List<Message> messages = receiveMessageResult.getMessages();
            if(messages.isEmpty()){
                System.out.println("No messages available");
            }else{
                for(Message msg: messages){
                    // From here message can be sent further to external systems
                    // Or persist messages to database
                    // But the messages needs to be deleted after consumption
                   // System.out.println(msg.getBody());

                    DeleteMessageRequest deleteMessageRequest = new DeleteMessageRequest(queueUrl, msg.getReceiptHandle());
                    sqsClient.deleteMessage(deleteMessageRequest);
                    System.out.println(msg.getBody() +" (Processed and deleted)");
                }
            }
        }
    }

}
