package com.rayfocus.sqs.app;

import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.model.GetQueueUrlRequest;
import com.amazonaws.services.sqs.model.GetQueueUrlResult;
import com.amazonaws.services.sqs.model.SendMessageBatchRequest;
import com.amazonaws.services.sqs.model.SendMessageBatchRequestEntry;
import com.rayfocus.sqs.client.SQSClientProvider;
import com.rayfocus.sqs.util.SQSUtil;

import java.util.List;

public class Producer {

    private static final String SQS_INPUTQ = "SQS_INPUTQ";
    private static final int BATCH_COUNT = 10;
    private static final int MAX_MESSAGES = 50;

    public static void main(String[] args) {
        Producer producer = new Producer();
        // Send message to SQS_INPUTQ
        //producer.sendMessageTo(SQS_INPUTQ);
        producer.sendBatchMessages(SQS_INPUTQ);
    }

    /* Send messages to Queue */
    public void sendMessageTo(String queueName){
        AmazonSQS sqsClient = SQSClientProvider.getForProfile("sqs_demo");
        String queueUrl = SQSUtil.getQueueUrlFor(queueName);
        String message = SQSUtil.getSimpleTextMessage();
        sqsClient.sendMessage(queueUrl, message);
        System.out.println("Message sent :: "+message);
    }

    public void sendMessageTo(String queueName, String message){
        AmazonSQS sqsClient = SQSClientProvider.getForProfile("sqs_demo");
        String queueUrl = SQSUtil.getQueueUrlFor(queueName);
        sqsClient.sendMessage(queueUrl, message);
        System.out.println("Message sent :: "+message);
    }

    /* Send batch messages to Queue */
    public void sendBatchMessages(String queueName){
        AmazonSQS sqsClient = SQSClientProvider.getForProfile("sqs_demo");
        String queueUrl = SQSUtil.getQueueUrlFor(queueName);
        int numOfMessages = 0;
        int numOfMessagesToAdd = 0;
        int maxMessages = MAX_MESSAGES;
        while(numOfMessages < maxMessages){
            if((numOfMessages + BATCH_COUNT) < maxMessages){
                numOfMessagesToAdd = BATCH_COUNT;
            }else{
                numOfMessagesToAdd = maxMessages - numOfMessages;
            }

            List<SendMessageBatchRequestEntry> batchMessages = SQSUtil.getBatchMessages(numOfMessages, numOfMessagesToAdd);
            if(!batchMessages.isEmpty()){
                SendMessageBatchRequest sendMessageBatchRequest =
                        new SendMessageBatchRequest(queueUrl, batchMessages);
                sqsClient.sendMessageBatch(sendMessageBatchRequest);
                numOfMessages = numOfMessages+numOfMessagesToAdd;
                System.out.println(numOfMessagesToAdd+" messages sent, Total messages in Queue: "+numOfMessages);
            }
        }
    }
}
