package com.rayfocus.sqs.lambda;

import com.amazonaws.services.lambda.runtime.Context;
import com.amazonaws.services.lambda.runtime.RequestHandler;
import com.amazonaws.services.lambda.runtime.events.SQSEvent;
import com.amazonaws.services.sqs.model.DeleteMessageRequest;
import com.rayfocus.sqs.client.SQSClientProvider;
import com.rayfocus.sqs.util.Message;
import com.rayfocus.sqs.util.SQSUtil;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.IOException;

public class SQSEventHandler implements RequestHandler<SQSEvent, Void> {
    /**
     * Handles a Lambda Function request for SQS event
     *
     * @param input   The Lambda Function input
     * @param context The Lambda execution environment context object.
     * @return The Lambda Function output
     */
    @Override
    public Void handleRequest(SQSEvent input, Context context) {
        System.out.println("SQS event message ::"+input);
        System.out.println("\n*************Receiving messages in Lambda event*************\n");
        for(SQSEvent.SQSMessage msg : input.getRecords()){
            String messageBody = msg.getBody();
            // Parse message object
            ObjectMapper objectMapper = new ObjectMapper();
            try {
                Message messageObj = objectMapper.readValue(messageBody, Message.class);
                System.out.println("Message number ==> "+messageObj.getId());
                if(messageObj.getId() == 3){
                    // For Test, if messageId == 3 move it to DLQ
                    moveToDLQ(messageBody, msg.getReceiptHandle());
                }else{
                    System.out.println(messageBody +" (processed)");
                }
            } catch (IOException e) {
                // Move the exceptional messages to DLQ
                System.out.println("Exception while processing..moving to DLQ");
                moveToDLQ(messageBody, msg.getReceiptHandle());
            }
        }
        return null;
    }

    private void moveToDLQ(String messageBody, String receiptHandle ){
        System.out.println("Sending message to DLQ");
        SQSClientProvider.get().sendMessage("SQS_INPUT_DLQ",messageBody);
        DeleteMessageRequest deleteMessageRequest = new DeleteMessageRequest(SQSUtil.getQueueUrlFor("SQS_INPUTQ"),
                receiptHandle);
        SQSClientProvider.get().deleteMessage(deleteMessageRequest);
    }
}
