package com.rayfocus.sqs.util;

import com.amazonaws.services.sqs.model.*;
import com.rayfocus.sqs.client.SQSClientProvider;

import java.util.ArrayList;
import java.util.List;

public class SQSUtil {

    public static String getQueueUrlFor(String queueName){
        GetQueueUrlRequest queueUrlRequest = new GetQueueUrlRequest(queueName);
        GetQueueUrlResult getQueueUrlResult =
                SQSClientProvider.getForProfile("sqs_demo").
                        getQueueUrl(queueUrlRequest);
        System.out.println("Queue URL for "+queueName+" :: "+getQueueUrlResult.getQueueUrl());
        return getQueueUrlResult.getQueueUrl();
    }

    public static CreateQueueRequest configureDLQ(CreateQueueRequest create_queue_request, String dlqName){
        String queueUrl = getQueueUrlFor(dlqName);
        GetQueueAttributesRequest getQueueAttributesRequest = new GetQueueAttributesRequest(queueUrl)
                .withAttributeNames("QueueArn");
        GetQueueAttributesResult queueAttributesResult = SQSClientProvider.getForProfile("sqs_demo").getQueueAttributes(getQueueAttributesRequest);
        String dlqARN = "";
        if(queueAttributesResult.getAttributes().keySet().contains("QueueArn")){
            dlqARN = queueAttributesResult.getAttributes().get("QueueArn");
        }
        String redrivePolicyVal = "{\"deadLetterTargetArn\":\"".concat(dlqARN).concat("\",\"maxReceiveCount\":20}");
        create_queue_request.addAttributesEntry("RedrivePolicy",redrivePolicyVal);
        return create_queue_request;
    }

    public static String getSimpleTextMessage(){
        return "{\"id\":1, \"data\": \"Simple Text Message\" }";
    }

    public static List<SendMessageBatchRequestEntry> getBatchMessages(int startIndex, int batchCount){
        List<SendMessageBatchRequestEntry> batchMessageEntries = new ArrayList<>();
        SendMessageBatchRequestEntry batchMessage = null;
        String messageBody = null;
        for(int id=startIndex+1; id<=(startIndex+batchCount); id++){
            messageBody = "{\"id\": "+id+", \"data\": \"Batch Message number "+id+"\" }";
            batchMessage = new SendMessageBatchRequestEntry("id"+Integer.toString(id), messageBody);
            batchMessageEntries.add(batchMessage);
        }
        return batchMessageEntries;
    }

}
